module BuildEsp where

import General
import Dictionary
import Print
import TypesEsp
import RulesEsp

-- combinators for lexicograph 
article :: Article -> Gender -> Entry
article t g = entryI t [prValue g]

article1 :: Article -> AGender -> Entry
article1 t g = entryI t [prValue g]
  
noun :: Noun -> Gender -> Entry
noun t g = entryI t [prValue g]

noun1 :: Noun1 -> Entry
noun1 = entry
   
adjective :: Adjective -> Entry
adjective = entry
   
iAdjective :: IrregAdjective -> Entry
iAdjective = entry

adverb :: Adverb -> Entry
adverb = entry

iAdverb :: IrregAdverb -> Entry
iAdverb = entry
   
verb :: Verb -> Entry
verb = entry
   
mkPN :: PN -> Gender -> Entry
mkPN pn g = entryI pn [prValue g] 
 
pnMasc :: String -> Entry
pnMasc s = mkPN (const (mkStr s)) Masc   
   
pnFem :: String -> Entry
pnFem s = mkPN (const (mkStr s)) Fem   
   
interj :: String -> Entry
interj s = entry f
  where f :: Interj
        f = const (mkStr s)

prep :: String -> Entry   
prep  s = entry f
  where f :: Prep
        f = const (mkStr s)

conj :: String -> Entry
conj s = entry f
  where f :: Prep
        f = const (mkStr s)

indefPron :: String -> Entry
indefPron s = entry f
  where f :: IndefPron
        f = const (mkStr s)

indefUni  :: IndefPron1 -> Entry
indefUni   = entry

demPron   :: DemoPron -> Entry
demPron    = entry

relaPron  :: RelPron -> Entry
relaPron   = entry

interPron :: IntPron -> Entry
interPron  = entry

nu        :: Numb -> Entry
nu         = entry

order     :: Order -> Entry
order      = entry

-- the commonest cases: all these take one string, the "dictionary form"   
   
nounMasc s = noun s Masc   
nounFem  s = noun s Fem   
   
n1  = nounMasc . normNoun   
n2  = nounFem  . normNoun   
n3  = nounMasc . noNumNoun   
n4  = nounFem  . noNumNoun   
n5  = nounMasc . noPlural   
n6  = nounFem  . noPlural   
n7  = nounMasc . noSing   
n8  = nounFem  . noSing 
n9  = noun1    . nounGen  
n10 = nounFem  . normNoun                 -- fem noun with masc art (agua)   
n11 = nounMasc . plurAccNoun
n12 = nounFem  . plurAccNoun
n13 = nounMasc . forNoun
n14 = nounFem  . forNoun
n15 = nounMasc . irregNoun
n16 = nounFem  . irregNoun

artMasc s = article s Masc   
artFem  s = article s Fem 

art1Masc  s = article1 s Masc1   
art1Fem   s = article1 s Fem1 
art1Neutr s = article1 s Neutr   

indefArt  = artMasc . artIndef
indefArt1 = artFem . artIndef

defArt  = art1Masc . artDef
defArt1 = art1Fem . artDef
defArt2 = art1Neutr . artDef
 

a1 = adjective . mkAdj1   
  
a3 = adjective . mkAdj3   
a4 = adjective . noNumAdj   
a5 = adjective . mkAdj1   
aIrreg = iAdjective . mkIrregAdj 

iadv :: String -> Entry
iadv s = iAdverb $ const (mkStr s)

adv :: String -> Entry
adv s  = adverb $ const (mkStr s)

v1    = verb . mkArVerb   
v2    = verb . mkErVerb   
v3    = verb . mkIrVerb   
vrefl = verb . mkVerbRefl   
v4    = verb . mkVerbSer
v5    = verb . mkVerbEstar
v6    = verb . mkVerbHaber
v7    = verb . mkVerbTener
--vref7 = verb . mkRefTener	          -- not done!
v8    = verb . mkVerbRomper
v9    = verb . mkVerbDefect
v10   = verb . mkVerbAbrir
v11   = verb . mkVerbAbolir
v12   = verb . mkVerbActuar
vref12   = verb . mkRef12
v13   = verb . mkVerbAdquirir
v14   = verb . mkVerbAhincar
v15   = verb . mkVerbAndar
v16   = verb . mkVerbAsir
vref16   = verb . mkRef16
v17   = verb . mkVerbAvergonzar
v18   = verb . mkVerbAveriguar
v19   = verb . mkVerbBruir
v20   = verb . mkVerbCaer
v21   = verb . mkVerbCocer
v22   = verb . mkVerbColgar
v23   = verb . mkVerbDar
v24   = verb . mkVerbDecir
v25   = verb . mkVerbDesosar
v26   = verb . mkVerbDormir
v27   = verb . mkVerbElegir
v28   = verb . mkVerbEncontrar
v29   = verb . mkVerbErguir
v30   = verb . mkVerbHacer
v31   = verb . mkVerbInfluir
v32   = verb . mkVerbMecer
v33   = verb . mkVerbOr
v34   = verb . mkVerbOler
v35   = verb . mkVerbAcaecer
v36   = verb . mkVerbPedir
v37   = verb . mkVerbAbarse
v38   = verb . mkVerbPoder

v40   = verb . mkVerbPredecir
v41   = verb . mkVerbProducir
v42   = verb . mkVerbProhibir
v43   = verb . mkVerbRaer
v44   = verb . mkVerbRer
v45   = verb . mkVerbSalir
v46   = verb . mkVerbSoler
v47   = verb . mkVerbVer
v48   = verb . mkVerbPrender
v49   = verb . mkVerbLlover

iuPron   = indefUni  . mkIndefPron1 
demoPron = demPron   . mkDemoPron
relPron  = relaPron  . mkRelPron  
intPron  = interPron . mkIntPron 

n        = nu . mkNumbers
o        = order . mkOrder
--       

-- Instances of Dict

instance Dict NounForm
  where category _ = "Noun"

instance Dict NounForm1
  where category _ = "Noun"

instance Dict PNForm
  where category _ = "Personal Noun"

instance Dict AdjForm where
    category _ = "Adjective"
   
instance Dict IrregAdjForm where
  category _ = "Adjective"

instance Dict VerbForm where
    category _ = "Verb"
   
instance Dict ArtForm where
  category _ = "Article"

instance Dict PrepForm where
  category _ = "Preposition"

instance Dict InterjForm where
  category _ = "Interjection"

instance Dict ConjForm where
  category _ = "Conjunction"

instance Dict PersPronForm where
    category _ = "Pers. Pronoun"

instance Dict ReflPronForm where
    category _ = "Ref. Pronoun"

instance Dict PossPronForm where
    category _ = "Poss. Pronoun"

instance Dict DemoPronForm where
    category _ = "Dem. Pronoun"

instance Dict RelPronForm where
    category _ = "Rel. Pronoun"

instance Dict IntPronForm where
    category _ = "Int. Pronoun" 

instance Dict IndefPronForm1 where
  category _ = "Indef. Pronoun"

instance Dict IrregAdvForm where
  category _ = "Irreg. Pronoun"

instance Dict AdverbForm where
  category _ = "Irreg. Pronoun"

instance Dict NumForm where
    category _ = "Cardinal"

instance Dict OrdForm where
    category _ = "Ordinal"
    
instance Dict IndefPronForm where
    category _ = "Indefinite Pronoun"
