
module RulesEsp where

import General
import TypesEsp
import Print 

-- Nouns   
   
-- To form plural endings -s or -es   
mkNumber,mkNumber1,mkNumber2,mkNumber3 :: Number -> String -> String   
mkNumber n s = case n of   
  Sg -> s   
  Pl -> case (last s) of   
	v | isVowel v  -> if v == '' then (s ++ "s " ++ s ++ "es")
			  else s ++ "s"     
        'z'  -> (tk 1 (checkAcc s)) ++ "ces"
        _    -> (checkAcc s) ++ "es"
mkNumber1 n s = case n of   
  Sg -> s   
  Pl -> if isVowel (last s) then plurAcc s ++ "s"
	else plurAcc s ++ "es"
mkNumber2 n s = case n of   
  Sg -> s   
  Pl -> s ++ "s"
mkNumber3 n s = case n of    
  Sg -> s    
  Pl -> (checkAcc s) ++ "es" 
   
-- To change the accent in plural   
checkAcc,plurAcc :: String -> String   
checkAcc s = buz ++  ++ n where   
  buz = tk 2 s   
  n = drop (length s - 1) s   
   = if accVowel (last (tk 1 s)) then   
          case (last (tk 1 s)) of   
                  '' -> "a"   
                  '' -> "e"   
                  '' -> "i"   
                  '' -> "o"   
                  '' -> "u"   
      else [(last (tk 1 s))] 
plurAcc s   = j ++  ++ ven where   
  (j,o,ven) = firstVowel s
           = case o of
                  "a" -> ""
                  "e" -> ""   
                  "i" -> ""   
                  "o" -> ""   
                  "u" -> ""     

normNoun,noNumNoun,noPlural,noSing,plurAccNoun,forNoun,irregNoun :: String -> Noun
normNoun  s (NF n)   = mkStrWords $ mkNumber n s
noNumNoun s (NF n)   = mkStr s   
noPlural  s (NF n)   = mkStrWords $ case n of   
    Sg -> s   
    Pl -> []   
noSing s (NF n)      = mkStrWords $ case n of   
    Sg -> []   
    Pl -> s
plurAccNoun s (NF n) = mkStrWords $ mkNumber1 n s
forNoun s (NF n)     = mkStrWords $ mkNumber2 n s
irregNoun  s (NF n)  = mkStrWords $ mkNumber3 n s 

nounGen :: String -> Noun1 
nounGen s (NF1 n g e)  = mkStrWords $ mkNumber n $ mkEval s e g
   
--nounGen :: String -> Noun1
--nounGen s (NF1 n g)  = mkStrWords $ mkNumber n $ mkNounGen s g

mkNounGen :: String -> Gender -> String   
mkNounGen s g = case g of   
    Masc -> s   
    Fem  -> if isVowel (last s) then 
	      if last s == 'o'  then (tk 1 s) ++ "a"
	      else s
	    else (checkAcc s) ++ "a"

mkEval :: String -> Eval -> Gender -> String
mkEval s e g = case e of
    Reg -> mkNounGen s g
    Dim -> mkDim s1 g
    Aug -> mkAug s1 g
    Pej -> mkPej s1 g
    where 
      s2 = checkAcc s
      s1 = if oneSyll s2 then s2 ++ "e" ++ "c" 
	       else case (last s2) of
	          v | isVowel v  -> if v == 'e' then s2 ++ "c" else tk 1 s2  
	          'n'  -> s2 ++ "c" 
	          'r'  -> s2 ++ "c" 
                  _    -> s2

-- To form diminutive endings    
mkDim, mkAug, mkPej :: String -> Gender -> String    
mkDim s g = case g of     
    Masc -> s ++ "ito"     
    Fem  -> s ++ "ita"    
    
-- To form augmentative endings    
mkAug s g = case g of     
    Masc -> s ++ "n"     
    Fem  -> s ++ "ona"    
    
-- To form pejorative endings    
mkPej s g = case g of     
    Masc -> s ++ "ucho"     
    Fem  -> s ++ "ucha"

oneSyll :: String -> Bool
oneSyll mes = (m == m1) 
	      where (m,e,s)     = firstVowel mes
		    (m1, e1, s1)= stemVowel  mes  


-- Adjectives   

mkAdj1,mkAdj3,noNumAdj :: String -> Adjective   
mkAdj1 s (AF n g) = mkStr $ mkNumber n $ mkAdjGen1 s g   
mkAdj3 s (AF n g) = mkStr $ mkNumber n $ mkAdjGen3 s g 
noNumAdj s _      = mkStr s     

mkAdjGen1,mkAdjGen3 :: String -> Gender -> String   
mkAdjGen1 s g = case g of   
    Masc -> s   
    Fem  -> if (last s) == 'o' then (tk 1 s) ++ "a" 
            else checkAcc s ++ "a"
mkAdjGen3 s g = s   


-- Irregular adjectives   
mkIrregAdj :: String -> IrregAdjective   
mkIrregAdj grande (IAF grade n g) = mkStrWords $ case grade of   
    Posit   -> if n == Sg && g == Masc then 
	       	if grande == "grande"
		 then (mkNumber n (mkAdjGen3 grande g) ++ " " ++ gran)
	       	else  (mkNumber n (mkAdjGen1 grande g) ++ " " ++ gran) 
	       else if grande == "grande"
		      then mkNumber n (mkAdjGen3 grande g) 
	       	    else   mkNumber n (mkAdjGen1 grande g) 	
    Compar  ->  if mayor == "ms" then "ms" 
		else mkNumber n (mkAdjGen3 mayor  g)   
    Superl  ->  if mayor == "ms" then "ms" 
		else mkNumber n (mkAdjGen3 mayor  g)
    where   
      gran = case grande of   
        "grande"  -> "gran"   
        "bueno"   -> "buen"   
        "malo"    -> "mal" 
        "mucho"   -> "" 
        "pequeo" -> ""   
        "poco"    -> ""   
      mayor =  case grande of   
        "grande"  -> "mayor"   
        "bueno"   -> "mejor"   
        "malo"    -> "peor"
        "mucho"   -> "ms"
        "pequeo" -> "menor"  
        "poco"    -> "menos" 
  

-- Irregular adverbs   
mkIrregAdv :: String -> IrregAdverb   
mkIrregAdv bien (IAdvF g) = mkStrWords $ case g of   
    Posit   -> bien 
    Compar  -> mejor  
    Superl  -> mejor  
    where   
      mejor =  case bien of   
        "bien"   -> "mejor"   
        "mal"    -> "peor"   
        "muy"    -> "ms"
	"mucho"  -> "ms"    
	"poco"   -> "menos"
   
-- Verbs   
  
type Endings = (String, String, String, String, String, String)   
   
type TwoStemVP    = String -> String -> VPers -> String   
type ThreeStemVP  = String -> String -> String -> VPers -> String   
type FourStemVP   = String -> String -> String -> String -> VPers -> String   
   
type OneStemEnd   = String -> Endings -> Number -> Person -> String   
type TwoStemEnd   = String -> String -> Endings -> Number -> Person -> String   
type ThreeStemEnd = String -> String -> String -> Endings -> Number -> Person -> String   
type FourStemEnd  = String -> String -> String -> String -> Endings -> Number -> Person -> String 
 
----------------------------------------------------------------------     
 
--v1; Regular -ar-verbs (group 5)   
mkArVerb :: String -> Verb   
mkArVerb s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPAr s tmnp   
   
mkVIAr :: String -> VImpers -> String   
mkVIAr s vim = case vim of   
    Inf    -> s   
    Ger    -> (stem s) ++ "ando"   
    Part   -> (stem s) ++ "ado"   
   
mkVPAr :: String -> VPers -> String   
mkVPAr hablar vp = case vp of   
    Pres Ind n p   -> mkEndings  habl presIndEndAr n p   
    Pres Subj n p  -> mkEndings  habl presSubjEndAr n p   
    Past Ind n p   -> mkEndings  habl pastIndEndAr n p   
    Past Subj n p  -> pastSubjAr habl n p   
    Pret n p       -> mkEndings  habl pretEndAr n p   
    Fut Ind n p    -> mkEndings  hablar futIndEnd n p   
    Fut Subj n p   -> mkEndings  habl futSubjEndAr n p   
    Cond n p       -> mkEndings  hablar condEnd n p   
    Imp n p        -> mkImpEnd   habl impEndAr n p
    Pass n g       -> mkPass    (habl ++ "ado") n g  
    where habl = stem hablar   
   
mkEndings :: OneStemEnd   
mkEndings habl (o, as, a, amos, is, an) n p = case n of   
    Sg -> case p of   
          P1 -> habl ++ o   
          P2 -> habl ++ as   
          P3 -> habl ++ a   
    Pl -> case p of   
          P1 -> habl ++ amos   
          P2 -> habl ++ is   
          P3 -> habl ++ an   
   
presIndEndAr,presSubjEndAr,pastIndEndAr,pastSubjEndAr,pastSubjEndAr1,pretEndAr,futIndEnd,futSubjEndAr,condEnd,impEndAr :: Endings
   
presIndEndAr   = ("o", "as", "a", "amos", "is", "an")   
presSubjEndAr  = ("e", "es", "e", "emos", "is", "en")   
pastIndEndAr   = ("aba", "abas", "aba", "bamos", "abais", "aban")   
pastSubjEndAr  = ("ara", "aras", "ara", "ramos", "arais", "aran")   
pastSubjEndAr1 = ("ase", "ases", "ase", "semos", "aseis", "asen")   
pretEndAr      = ("", "aste", "", "amos", "asteis", "aron")   
futIndEnd      = ("", "s", "", "emos", "is", "n")   
futSubjEndAr   = ("are", "ares", "are", "remos", "areis", "aren")   
condEnd        = ("a", "as", "a", "amos", "ais", "an")   
impEndAr       = ([], "a", "e", "emos", "ad", "en") 

pastSubjAr :: String -> Number -> Person -> String
pastSubjAr s n p = hablara ++ " " ++ hablase where
  hablara  = mkEndings s pastSubjEndAr n p
  hablase  = mkEndings s pastSubjEndAr1 n p 

mkImpEnd :: OneStemEnd   
mkImpEnd habl ([], e, a, amos, id, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> habl ++ e   
          P3 -> habl ++ a   
    Pl -> case p of   
          P1 -> habl ++ amos   
          P2 -> habl ++ id   
          P3 -> habl ++ an 

mkPass s n g = mkNumber n $ mkAdjGen1 s g      
   
----------------------------------------------------------------------   
   
--v2; Regular -er-verbs (group 6)   
mkErVerb :: String -> Verb   
mkErVerb s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPEr s tmnp   
   
mkVIEr :: String -> VImpers -> String   
mkVIEr s vim = case vim of   
    Inf    -> s   
    Ger    -> (stem s) ++ "iendo"   
    Part   -> (stem s) ++ "ido"   
   
mkVPEr :: String -> VPers -> String   
mkVPEr comer vp = case vp of   
    Pres Ind n p   -> mkEndings  com presIndEndEr n p   
    Pres Subj n p  -> mkEndings  com presSubjEndEr n p   
    Past Ind n p   -> mkEndings  com pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr com  n p   
    Pret n p       -> mkEndings  com  pretEndEr n p   
    Fut Ind n p    -> mkEndings  comer futIndEnd n p   
    Fut Subj n p   -> mkEndings  com futSubjEndEr n p   
    Cond n p       -> mkEndings  comer condEnd n p   
    Imp n p        -> mkImpEnd   com impEndEr n p
    Pass n g       -> mkPass    (com ++ "ido") n g     
    where com = stem comer   
   
presIndEndEr,presSubjEndEr,pastIndEndEr,pastSubjEndEr,pastSubjEndEr1, 
 pretEndEr,futSubjEndEr,impEndEr :: Endings   
presIndEndEr   = ("o", "es", "e", "emos", "is", "en")   
presSubjEndEr  = ("a", "as", "a", "amos", "is", "an")   
pastIndEndEr   = ("a", "as", "a", "amos", "ais", "an")   
pastSubjEndEr  = ("iera", "ieras", "iera", "iramos", "ierais", "ieran")   
pastSubjEndEr1 = ("iese", "ieses", "iese", "isemos", "ieseis", "iesen")   
pretEndEr      = ("", "iste", "i", "imos", "isteis", "ieron")   
futSubjEndEr   = ("iere", "ieres", "iere", "iremos", "iereis", "ieren")   
impEndEr       = ([], "e", "a", "amos", "ed", "an")   
   
pastSubjEr :: String -> Number -> Person -> String   
pastSubjEr s n p = comiera ++ " " ++ comiese where   
  comiera  = mkEndings s pastSubjEndEr n p   
  comiese  = mkEndings s pastSubjEndEr1 n p   

----------------------------------------------------------------------   
   
--v3; Regular -ir-verbs (group 7)   
mkIrVerb :: String -> Verb   
mkIrVerb s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPIr s tmnp   
   
mkVPIr :: String -> VPers -> String   
mkVPIr vivir vp = case vp of   
    Pres Ind n p   -> mkEndings  viv presIndEndIr n p   
    Pres Subj n p  -> mkEndings  viv presSubjEndEr n p   
    Past Ind n p   -> mkEndings  viv pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr viv n p   
    Pret n p       -> mkEndings  viv pretEndEr n p   
    Fut Ind n p    -> mkEndings  vivir futIndEnd n p   
    Fut Subj n p   -> mkEndings  viv futSubjEndEr n p   
    Cond n p       -> mkEndings  vivir condEnd n p   
    Imp n p        -> mkImpEnd   viv impEndIr n p
    Pass n g       -> if vivir == "pudrir" then mkPass "podrido" n g  
		      else mkPass (viv ++ "ido") n g        
    where viv = stem vivir   
  
presIndEndIr,impEndIr :: Endings   
presIndEndIr = ("o", "es", "e", "imos", "s", "en")   
impEndIr     = ([], "e", "a", "amos", "id", "an")   
   
---------------------------------------------------------------------   
   
--vref; Reflexive regular -ar, -er and -ir-verbs   
mkVerbRefl :: String -> Verb   
mkVerbRefl s = case stemmed of   
    ar | arVerb ar -> mkArVerb stemmed `except` [(VI Inf, s)]   
    er | erVerb er -> mkErVerb stemmed `except` [(VI Inf, s)]   
    ir | irVerb ir -> mkIrVerb stemmed `except` [(VI Inf, s)]   
    where stemmed = stem s   
   
---------------------------------------------------------------------   
   
-- Irregular verbs   
   
-- v4: ser 1 (aux. verb), ir 46   
mkVerbSer :: String -> Verb   
mkVerbSer s vf = mkStrWords $ case vf of   
    VI f    -> mkVISer s f   
    VP tmnp -> mkVPSer s s1 s2 s3 tmnp   
    where   
      s1 = if s == "ser" then "se" else "vay"   
      s2 = if s == "ser" then "er" else "ib"   
      s3 = "fu"   
   
mkVISer :: String -> VImpers -> String   
mkVISer s vim = case vim of   
    Inf    -> s   
    Ger    -> if s == "ser" then "siendo" else "yendo"   
    Part   -> if s == "ser" then "sido" else "ido"
   
mkVPSer :: FourStemVP   
mkVPSer ser se er fu vp = case vp of   
    Pres Ind n p   -> if ser == "ser" then mkNPForm presIndSer n p   
                      else mkNPForm presIndIr n p   
    Pres Subj n p  -> mkEndings   se presSubjEndEr n p   
    Past Ind n p   -> mkPastInd   er n p                 
    Past Subj n p  -> pastSubjSer fu n p   
    Pret n p       -> mkEndings   fu pretEndSer n p   
    Fut Ind n p    -> mkEndings   ser futIndEnd n p   
    Fut Subj n p   -> mkEndings   fu futSubjEndSer n p   
    Cond n p       -> mkEndings   ser condEnd n p   
    Imp n p        -> if ser == "ser" then mkNPForm impSer n p   
                      else mkNPForm impIr n p
    Pass n g       -> if ser == "ser" then mkPass "sido" n g 
		      else mkPass "ido" n g        
    where s = if ser == "ser" then "s" else ""   
   
mkNPForm :: Endings -> Number -> Person -> String   
mkNPForm (soy, eres, es, somos, sois, son) n p = case n of   
    Sg -> case p of   
          P1 -> soy   
          P2 -> eres   
          P3 -> es   
    Pl -> case p of   
          P1 -> somos   
          P2 -> sois   
          P3 -> son   
   
presIndSer,presIndIr,pastSubjEndSer,pastSubjEndSer1,pretEndSer, 
 futSubjEndSer,impIr,impSer :: Endings   
presIndSer      = ("soy", "eres", "es", "somos", "sois", "son")   
presIndIr       = ("voy", "vas", "va", "vamos", "vais", "van")   
pastSubjEndSer  = ("era", "eras", "era", "ramos", "erais", "eran")   
pastSubjEndSer1 = ("ese", "eses", "ese", "semos", "eseis", "esen")   
pretEndSer      = ("i", "iste", "e", "imos", "isteis", "eron")   
futSubjEndSer   = ("ere", "eres", "ere", "remos", "ereis", "eren")   
impSer          = ([], "s", "sea", "seamos", "sed", "sean")   
impIr           = ([], "ve", "vaya", "vamos vayamos", "id", "vayan") 

mkPastInd :: String -> Number -> Person -> String   
mkPastInd er n p = case n of   
    Sg -> case p of   
          P1 -> er ++ "a"   
          P2 -> er ++ "as"   
          P3 -> er ++ "a"   
    Pl -> case p of   
          P1 -> accent er ++ "amos"   
          P2 -> er ++ "ais"   
          P3 -> er ++ "an"   
   
pastSubjSer :: String -> Number -> Person -> String   
pastSubjSer s n p = fuera ++ " " ++ fuese where   
  fuera  = mkEndings s pastSubjEndSer n p   
  fuese  = mkEndings s pastSubjEndSer1 n p     
   
---------------------------------------------------------------------   
   
--v5; estar 2   
mkVerbEstar :: String -> Verb   
mkVerbEstar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPEstar s s1 tmnp   
    where s1 = (stem s) ++ "uv"   
   
mkVPEstar :: TwoStemVP   
mkVPEstar estar estuv vp = case vp of   
    Pres Ind n p   -> mkEndings  est presIndEndEstar n p   
    Pres Subj n p  -> mkEndings  est presSubjEndEstar n p   
    Past Ind n p   -> mkEndings  est pastIndEndAr n p   
    Past Subj n p  -> pastSubjEr estuv n p   
    Pret n p       -> mkEndings  estuv pretEndHaber n p   
    Fut Ind n p    -> mkEndings  estar futIndEnd n p   
    Fut Subj n p   -> mkEndings  estuv futSubjEndEr n p   
    Cond n p       -> mkEndings  estar condEnd n p   
    Imp n p        -> mkImpEnd   est impEstar n p 
    Pass n g       -> mkPass    (est ++ "ado") n g    
    where   
      est = stem estar   
   
mkEnd1 :: TwoStemEnd   
mkEnd1 sab s (o, es, e, emos, is, en) n p = case n of   
    Sg -> case p of   
          P1 -> s   
          P2 -> sab ++ es   
          P3 -> sab ++ e   
    Pl -> case p of   
          P1 -> sab ++ emos   
          P2 -> sab ++ is   
          P3 -> sab ++ en   
   
presIndEndEstar,presSubjEndEstar,impEstar :: Endings   
presIndEndEstar  = ("oy", "s", "", "amos", "is", "n")   
presSubjEndEstar = ("", "s", "", "emos", "is", "n")   
impEstar         = ([], "", "", "emos", "ad", "n")   
   
---------------------------------------------------------------------   
   
--v6; haber 3 (aux. verb), caber 19, saber 71   
mkVerbHaber :: String -> Verb   
mkVerbHaber s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPHaber s s1 s2 s3 tmnp   
    where   
      (pre, saber) = vPrefix 5 s   
      s1 = case saber of   
        "haber" -> "hay"   
        "caber" -> "quep"   
        "saber" -> pre ++ "sep"   
      s2 = case saber of   
        "haber" -> "hub"   
        "caber" -> "cup"   
        "saber" -> pre ++ "sup"   
      s3 = dropSecondLast s	   
   
mkVPHaber :: FourStemVP   
mkVPHaber haber hay hub habr vp = case vp of   
    Pres Ind n p   -> if haber == "haber" then mkNPForm presIndHaber n p   
                      else mkEnd1 hab sp1 presIndEndEr n p   
    Pres Subj n p  -> mkEndings  hay presSubjEndEr n p   
    Past Ind n p   -> mkEndings  hab pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr hub n p   
    Pret n p       -> mkEndings  hub pretEndHaber n p   
    Fut Ind n p    -> mkEndings  habr futIndEnd n p   
    Fut Subj n p   -> mkEndings  hub futSubjEndEr n p   
    Cond n p       -> mkEndings  habr condEnd n p   
    Imp n p        -> if haber == "haber" then []   
                      else mkImpEnd3 hab hay impEndEr n p
    Pass n g       -> mkPass    (hab ++ "ido") n g     
    where   
      hab = stem haber   
      (pre, saber) = vPrefix 5 haber   
      sp1 = if saber == "saber" then (pre ++ "s") else (hay ++ "o")   
   
presIndHaber,pretEndHaber :: Endings   
presIndHaber = ("he", "has", "ha hay", "hemos", "habis", "han")   
pretEndHaber = ("e", "iste", "o", "imos", "isteis", "ieron")   
   
mkImpEnd3 :: TwoStemEnd   
mkImpEnd3 sab sep ([], e, a, amos, ed, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> sab ++ e   
          P3 -> sep ++ a   
    Pl -> case p of   
          P1 -> sep ++ amos   
          P2 -> sab ++ ed   
          P3 -> sep ++ an   
   
---------------------------------------------------------------------   
   
--v7; tener 4, poner 60, venir 82   
mkVerbTener :: String -> Verb   
mkVerbTener s vf = mkStrWords $ case vf of   
    VI f    -> mkVITener s pf f   
    VP tmnp -> mkVPTener pf s s1 s2 s3 tmnp   
    where    
      (pre, tener) = vPrefix 5 s   
      pf = case tener of    
        "tener" -> pre ++ "tenido"    
        "poner" -> pre ++ "puesto"   
        "venir" -> pre ++ "venido"   
      s1 = if tener == "poner" then stem s    
           else dift (stem s)         
      s2 = case tener of    
        "tener" -> pre ++ "tuv"    
        "poner" -> pre ++ "pus"   
        "venir" -> pre ++ "vin"   
      s3 = (stem s) ++ "dr"   
   
mkVITener :: String -> String -> VImpers -> String   
mkVITener s pForm vim = case vim of   
    Inf    -> s   
    Ger    -> if s == "venir" then "viniendo"    
              else (stem s) ++ "iendo"   
    Part   -> pForm    
   
mkVPTener :: String -> FourStemVP   
mkVPTener partform tener tien tuv tendr vp = case vp of   
    Pres Ind n p   -> if irVerb tener then mkEnd4 ten tien presIndEndVenir n p   
                      else mkEnd4 ten tien presIndEndTener n p    
    Pres Subj n p  -> mkEndings  ten presSubjEndTener n p   
    Past Ind n p   -> mkEndings  ten pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr tuv n p   
    Pret n p       -> mkEndings  tuv pretEndHaber n p   
    Fut Ind n p    -> mkEndings  tendr futIndEnd n p   
    Fut Subj n p   -> mkEndings  tuv futSubjEndEr n p   
    Cond n p       -> mkEndings  tendr condEnd n p   
    Imp n p        -> if irVerb  tener then mkImpEnd ten impEndVenir n p   
                      else mkImpEnd ten impEndTener n p 
    Pass n g       -> mkPass     partform n g     
    where ten = stem tener   
   
mkEnd4 :: TwoStemEnd     
mkEnd4 ten tien (go, es, e, emos, is, en) n p = case n of   
    Sg -> case p of   
          P1 -> ten  ++ go   
          P2 -> tien ++ es   
          P3 -> tien ++ e   
    Pl -> case p of   
          P1 -> ten  ++ emos   
          P2 -> ten  ++ is   
          P3 -> tien ++ en   
   
presIndEndVenir,presIndEndTener,presSubjEndTener,impEndVenir,impEndTener :: Endings   
presIndEndVenir  = ("go", "es", "e", "imos", "s", "en")   
presIndEndTener  = ("go", "es", "e", "emos", "is", "en")   
presSubjEndTener = ("ga", "gas", "ga", "gamos", "gis", "gan")   
impEndVenir      = ([], "", "ga", "gamos", "id", "gan")   
impEndTener      = ([], "", "ga", "gamos", "ed", "gan")   
   
---------------------------------------------------------------------   
   
--v8; romper 6, volver , pudrir 59  - Irregular participle   
mkVerbRomper :: String -> Verb   
mkVerbRomper s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregPartEr s pf f   
    VP tmnp -> mkVPVolver pf s s1 tmnp   
    where 
      pf = case s of 
           "romper" -> "roto"
	   "volver" -> "vuelto"
	   "pudrir" -> "podrido"
      s1 = if s == "volver" then dift (stem s) else stem s
   
mkVIIrregPartEr :: String -> String -> VImpers -> String   
mkVIIrregPartEr s partForm vim = case vim of   
    Inf    -> if s == "pudrir" then s ++ " podrir" else s   
    Ger    -> (stem s) ++ "iendo"   
    Part   -> partForm   

mkVPVolver :: ThreeStemVP   
mkVPVolver partForm volver vuelv vp = case vp of   
    Pres Ind n p   -> if erVerb volver   
                      then mkEnd9 volv vuelv presIndEndEr n p   
                      else mkEnd9 volv vuelv presIndEndIr n p   
    Pres Subj n p  -> mkEnd9      volv vuelv presSubjEndEr n p   
    Past Ind n p   -> mkEndings   volv pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr  volv n p   
    Pret n p       -> mkEndings   volv pretEndEr n p   
    Fut Ind n p    -> mkEndings   volver futIndEnd n p   
    Fut Subj n p   -> mkEndings   volv futSubjEndEr n p   
    Cond n p       -> mkEndings   volver condEnd n p   
    Imp n p        -> if erVerb   volver   
                      then mkImpEnd9 volv vuelv impEndEr n p   
                      else mkImpEnd9 volv vuelv impEndIr n p   
    Pass n g       -> mkPass      partForm n g    
    where volv = stem volver   

---------------------------------------------------------------------   
   
--v9; descolorir 7, denegrir 7, usucapir 7 - Defect verbs   
mkVerbDefect :: String -> Verb   
mkVerbDefect s vf = mkStrWords $ case vf of   
    VI f    -> if (s == "denegrir" || s == "usucapir")    
	       then mkVIEr s f   
               else mkVIDefect s f    
    VP tmnp -> []   
   
mkVIDefect :: String -> VImpers -> String   
mkVIDefect s vim = case vim of   
    Inf    -> s   
    Ger    -> []   
    Part   -> if (s == "adir" || s == "soler") then []    
	      else (stem s) ++ "ido"    
   
---------------------------------------------------------------------     
   
--v10; abrir 7, ... - Irregular participle   
mkVerbAbrir :: String -> Verb   
mkVerbAbrir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregPartEr s pf f   
    VP tmnp -> mkVPIr s tmnp   
    where   
      partStem = tk 3 s   
      pf = if (last partStem) == 'i' then (partStem ++ "to")   
           else (partStem ++ "ierto")   
   
---------------------------------------------------------------------   
   
--v11; abolir 8, balbucir 8, ...   
-- Defect verbs   
mkVerbAbolir :: String -> Verb   
mkVerbAbolir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPAbolir s tmnp   
   
mkVPAbolir :: String -> VPers -> String   
mkVPAbolir abolir vp = case vp of   
    Pres Ind n p   -> if abolir == "balbucir"    
                      then mkEnd24 abol [] presIndEndBalbucir n p    
		      else mkEnd9  abol [] presIndEndAbolir n p   
    Pres Subj n p  -> []   
    Past Ind n p   -> mkEndings    abol pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr   abol n p   
    Pret n p       -> mkEndings    abol pretEndEr n p   
    Fut Ind n p    -> mkEndings    abolir futIndEnd n p   
    Fut Subj n p   -> mkEndings    abol futSubjEndEr n p   
    Cond n p       -> mkEndings    abolir condEnd n p   
    Imp n p 	   -> case p of   
		      P1 -> []   
		      P2 -> if abolir == "balbucir" then mkEndBalbucir abol n    
                            else if n == Pl then abol ++ "id"    
                                 else []   
		      P3 -> []
    Pass n g       -> mkPass      (abol ++ "ido") n g
    where abol = stem abolir   
   
presIndEndBalbucir,presIndEndAbolir :: Endings   
presIndEndBalbucir = ([], "es", "e", "imos", "s", "en")   
presIndEndAbolir   = ([], [], [], "imos", "s", [])   
   
mkEndBalbucir :: String -> Number -> String   
mkEndBalbucir balbuc n = case n of   
			 Sg -> balbuc ++ "e"   
			 Pl -> balbuc ++ "id"   
   
---------------------------------------------------------------------   
   
--v12; actuar 9, airar 12, aullar 15, guiar 43  
-- Verbs with an accent   
mkVerbActuar :: String -> Verb   
mkVerbActuar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPActuar s s1 tmnp   
    where s1 = accent (stem s)   
   
mkVPActuar :: TwoStemVP   
mkVPActuar actuar act vp = case vp of   
    Pres Ind n p   -> mkEnd9     actu act presIndEndAr n p   
    Pres Subj n p  -> mkEnd9     actu act presSubjEndAr n p   
    Past Ind n p   -> mkEndings  actu pastIndEndAr n p   
    Past Subj n p  -> pastSubjAr actu n p   
    Pret n p       -> mkEndings  actu pretEndAr n p   
    Fut Ind n p    -> mkEndings  actuar futIndEnd n p   
    Fut Subj n p   -> mkEndings  actu futSubjEndAr n p   
    Cond n p       -> mkEndings  actuar condEnd n p   
    Imp n p        -> mkImpEnd9  actu act impEndAr n p
    Pass n g       -> mkPass    (actu ++ "ado") n g   
    where actu = stem actuar   
   
-- Stem s  is used in 1 and 2 pers plur   
-- Stem s1 is used in every other case   
mkEnd9 :: TwoStemEnd   
mkEnd9 actu act (o, as, a, amos, is, an) n p = case n of   
    Sg -> case p of   
          P1 -> act ++ o   
          P2 -> act ++ as   
          P3 -> act ++ a   
    Pl -> case p of   
          P1 -> actu ++ amos   
          P2 -> actu ++ is   
          P3 -> act ++ an   
   
-- stem s  is used in 1 and 2 pers plur   
-- stem s1 is the stem with an accent   
-- used in 2 and 3 pers sing and 3 pers plur   
mkImpEnd9 :: TwoStemEnd   
mkImpEnd9 actu act ([], e, a, amos, id, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> act ++ e   
          P3 -> act ++ a   
    Pl -> case p of   
          P1 -> actu ++ amos   
          P2 -> actu ++ id   
          P3 -> act ++ an   
   
---------------------------------------------------------------------   

--vref12; Reflexive irregular -ar, -er and -ir-verbs v12, v13, v36  
mkRef12 :: String -> Verb   
mkRef12 s = case stemmed of   
    ar | arVerb ar -> mkVerbActuar   stemmed `except` [(VI Inf, s)]   
    er | erVerb er -> mkVerbAdquirir stemmed `except` [(VI Inf, s)]   
    ir | irVerb ir -> mkVerbPedir    stemmed `except` [(VI Inf, s)]   
    where stemmed = stem s   
   
---------------------------------------------------------------------   
      
--v13; adquirir 10, defender 29, discernir 33, mover 50, volver 84    
-- (volver84 obs participformen)   
-- Verbs with a diphthong   
mkVerbAdquirir :: String -> Verb   
mkVerbAdquirir s vf = mkStrWords $ case vf of   
    VI f    -> if s == "volver" then mkVIIrregPartEr s "vuelto" f
	       else mkVIEr s f   
    VP tmnp -> mkVPProhibir s s1 tmnp   
    where s1 = dift (stem s)   
  
---------------------------------------------------------------------   
   
--v14; ahincar 11, enraizar 39   
-- Verbs with orthographical changes and accent   
mkVerbAhincar :: String -> Verb   
mkVerbAhincar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPColgar s s1 s2 s3 tmnp   
    where   
      s1 = accent (stem s)   
      s2 = orto4 (stem s)   
      s3 = orto4 (accent (stem s))   
   
---------------------------------------------------------------------   
   
-- airar 12 (see bgroup 9)   
   
---------------------------------------------------------------------   
   
--v15; andar 13   
mkVerbAndar :: String -> Verb   
mkVerbAndar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPAndar s s1 tmnp   
    where s1 = (stem s) ++ "uv"   
   
mkVPAndar :: TwoStemVP   
mkVPAndar andar anduv vp = case vp of   
    Pres Ind n p   -> mkEnd1     and sp1 presIndEndAr n p   
    Pres Subj n p  -> mkEndings  and presSubjEndAr n p   
    Past Ind n p   -> mkEndings  and pastIndEndAr n p   
    Past Subj n p  -> pastSubjEr anduv n p   
    Pret n p       -> mkEndings  anduv pretEndHaber n p   
    Fut Ind n p    -> mkEndings  andar futIndEnd n p   
    Fut Subj n p   -> mkEndings  anduv futSubjEndEr n p   
    Cond n p       -> mkEndings  andar condEnd n p   
    Imp n p        -> mkImpEnd   and impEndAr n p 
    Pass n g       -> mkPass    (and ++ "ado") n g  
    where   
      and = stem andar   
      sp1 = and ++ "o"   
   
---------------------------------------------------------------------   
   
--v16; asir 14, coger 23, conocer 25, delinquir 30,   
-- dirigir 32, distinguir 34, lucir 48,   
-- parecer 54, (placer 57 obs! extra forms)   
-- Verbs with orthographical changes   
mkVerbAsir :: String -> Verb   
mkVerbAsir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPAsir s s1 tmnp   
    where s1 = orto (stem s)   
   
mkVPAsir :: TwoStemVP   
mkVPAsir asir asg vp = case vp of   
    Pres Ind n p   -> if erVerb asir   
                      then mkEnd24 as asg presIndEndEr n p   
                      else mkEnd24 as asg presIndEndIr n p   
    Pres Subj n p  -> if placer == "placer" then placerExtra asg vp    
                      else mkEndings asg presSubjEndEr n p   
    Past Ind n p   -> mkEndings as pastIndEndEr n p   
    Past Subj n p  -> if placer == "placer" then placerExtra as vp    
                      else pastSubjEr as n p   
    Pret n p       -> if placer == "placer" then placerExtra as vp    
                      else mkEndings as pretEndEr n p   
    Fut Ind n p    -> mkEndings asir futIndEnd n p   
    Fut Subj n p   -> if placer == "placer" then placerExtra as vp    
                      else mkEndings as futSubjEndEr n p   
    Cond n p       -> mkEndings asir condEnd n p   
    Imp n p        -> if erVerb asir   
                      then mkImpEnd3 as asg impEndEr n p   
                      else mkImpEnd3 as asg impEndIr n p
    Pass n g       -> mkPass        (as ++ "ido") n g   
    where    
      as = stem asir   
      (_, placer) = vPrefix 6 asir   
   
placerExtra :: String -> VPers -> String   
placerExtra plac vp = case vp of    
    Pres Subj n p  -> if sing3 n p then (mkEndings plac presSubjEndEr n p) ++ " plegue plega"   
                      else mkEndings plac presSubjEndEr n p    
    Past Subj n p  -> if sing3 n p then (pastSubjEr plac n p) ++ " pluguiera pluguiese"   
		      else pastSubjEr plac n p   
    Pret n p       -> if p == P3 then   
		         if sing3 n p then (mkEndings plac pretEndEr n p) ++ " plugo"   
			 else (mkEndings plac pretEndEr n p) ++ " pluguieron"   
		      else mkEndings plac pretEndEr n p   
    Fut Subj n p   -> if sing3 n p then (mkEndings plac futSubjEndEr n p) ++ " pluguiere"   
		      else mkEndings plac futSubjEndEr n p   
   
---------------------------------------------------------------------   

--vref16; Reflexive irregular -ar, -er and -ir-verbs v16, v18, v19  
mkRef16 :: String -> Verb   
mkRef16 s = case stemmed of   
    ar | arVerb ar -> mkVerbAveriguar stemmed `except` [(VI Inf, s)]   
    er | erVerb er -> mkVerbAsir      stemmed `except` [(VI Inf, s)]   
    ir | irVerb ir -> if s == "translucirse" then mkVerbAsir stemmed `except` [(VI Inf, s)] 
		      else mkVerbBruir stemmed `except` [(VI Inf, s)]   
    where stemmed = stem s   
   
---------------------------------------------------------------------   
   
-- aullar 15 (see group 9)   
   
---------------------------------------------------------------------   
   
--v17; avergonzar 16   
-- Verb with orthographical changes and diphthong   
mkVerbAvergonzar :: String -> Verb   
mkVerbAvergonzar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPColgar s s1 s2 s3 tmnp   
    where   
      s1 = dift2 (stem s)   
      s2 = orto4 (stem s)   
      s3 = orto4 (dift2 (stem s))   
   
---------------------------------------------------------------------   
   
--v18; averiguar 17, cazar 21, pagar 53, sacar 72   
-- Verbs with orthographical changes   
mkVerbAveriguar :: String -> Verb   
mkVerbAveriguar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPColgar s (stem s) s1 s1 tmnp   
    where   
      s1 = orto4 (stem s)   
   
---------------------------------------------------------------------   
   
--v19; bruir 18, taer 78   
-- Verbs with orthographical changes   
mkVerbBruir :: String -> Verb   
mkVerbBruir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIBruir s f   
    VP tmnp -> mkVPBruir s tmnp   
   
mkVIBruir :: String -> VImpers -> String   
mkVIBruir s vim = case vim of   
    Inf    -> s   
    Ger    -> (stem s) ++ "endo"   
    Part   -> (stem s) ++ "ido"   
   
mkVPBruir :: String -> VPers -> String   
mkVPBruir bruir vp = case vp of   
    Pres Ind n p   -> if irVerb bruir then mkEndings bru presIndEndIr n p   
		      else mkEndings bru presIndEndEr n p   
    Pres Subj n p  -> mkEndings   bru presSubjEndEr n p   
    Past Ind n p   -> mkEndings   bru pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer bru n p   
    Pret n p       -> mkEndings   bru pretEndBruir n p   
    Fut Ind n p    -> mkEndings   bruir futIndEnd n p   
    Fut Subj n p   -> mkEndings   bru futSubjEndSer n p   
    Cond n p       -> mkEndings   bruir condEnd n p   
    Imp n p        -> if irVerb   bruir then mkImpEnd bru impEndIr n p   
		      else mkImpEnd bru impEndEr n p
    Pass n g       -> mkPass     (bru ++ "ido") n g   
    where bru = stem bruir   
   
pretEndBruir :: Endings   
pretEndBruir = ("", "iste", "", "imos", "isteis", "eron")   
   
---------------------------------------------------------------------   
   
-- caber 19 (see group 3)   
   
---------------------------------------------------------------------   
   
--v20; caer 20, creer 26, roer 70, traer 79   
mkVerbCaer :: String -> Verb   
mkVerbCaer s vf = mkStrWords $ case vf of   
    VI f    -> mkVICaer s f   
    VP tmnp -> mkVPCaer s s1 s2 tmnp    
    where   
      (pre, caer) = vPrefix 4 s    
      s1 = if caer == "caer" then (stem s) ++ "ig"   
           else stem s   
      s2 = stem s ++ "y"   
   
mkVICaer :: String -> VImpers -> String   
mkVICaer s vim = case vim of   
    Inf    -> s   
    Ger    -> stem s ++ "yendo"   
    Part   -> if s == "proveer" then (stem s) ++ "do" ++ " provisto"
	      else (stem s) ++ "do"   
   
mkVPCaer :: ThreeStemVP    
mkVPCaer caer caig cay vp = case vp of   
    Pres Ind n p   -> if caer == "roer" then if n == Sg && p == P1 
		                             then (mkEnd24 ca caig presIndEndEr n p) ++ " roigo" 
					     else mkEnd24 ca caig presIndEndEr n p   
		      else mkEnd24 ca caig presIndEndEr n p   
    Pres Subj n p  -> mkEndings   caig presSubjEndEr n p   
    Past Ind n p   -> mkEndings   ca pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer cay n p   
    Pret n p       -> mkEndings   ca pretEndCaer n p   
    Fut Ind n p    -> mkEndings   caer futIndEnd n p   
    Fut Subj n p   -> mkEndings   cay futSubjEndSer n p   
    Cond n p       -> mkEndings   caer condEnd n p   
    Imp n p        -> mkImpEnd3   ca caig impEndEr n p 
    Pass n g       ->if caer == "proveer" then (mkPass (ca ++ "do") n g) ++ (mkPass " provisto" n g)
		     else mkPass (ca ++ "do") n g
    where    
      ca = stem caer   
      (pre, traer) = vPrefix 4 caer    
   
pretEndCaer :: Endings    
pretEndCaer = ("", "ste", "y", "mos", "steis", "yeron")   
   
---------------------------------------------------------------------   
   
-- cazar 21 (see group 17)   
   
---------------------------------------------------------------------   
   
--v21; cocer 22   
-- Verb with orthographical changes and diphthong   
mkVerbCocer :: String -> Verb   
mkVerbCocer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPCocer s s1 s2 s3 tmnp   
    where   
      s1 = dift (stem s)   
      s2 = orto1 (stem s)   
      s3 = orto1 (dift (stem s))   
   
mkVPCocer :: FourStemVP   
mkVPCocer cocer cuec coz cuez vp = case vp of   
    Pres Ind n p   -> mkEnd22    coc cuec cuez presIndEndEr n p   
    Pres Subj n p  -> mkEnd9     coz cuez presSubjEndEr n p   
    Past Ind n p   -> mkEndings  coc pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr coc n p   
    Pret n p       -> mkEndings  coc pretEndEr n p   
    Fut Ind n p    -> mkEndings  cocer futIndEnd n p   
    Fut Subj n p   -> mkEndings  coc futSubjEndEr n p   
    Cond n p       -> mkEndings  cocer condEnd n p   
    Imp n p        -> mkImpEnd24 coc cuec coz cuez impEndEr n p
    Pass n g       -> mkPass    (coc ++ "ido") n g 
    where coc = stem cocer   
   
mkEnd22 :: ThreeStemEnd   
mkEnd22 coc cuec cuez (o, es, e, emos, is, en) n p = case n of   
    Sg -> case p of   
          P1 -> cuez ++ o   
          P2 -> cuec ++ es   
          P3 -> cuec ++ e   
    Pl -> case p of   
          P1 -> coc  ++ emos   
          P2 -> coc  ++ is   
          P3 -> cuec ++ en   
   
---------------------------------------------------------------------   
   
-- coger 23 (see group 14)   
   
---------------------------------------------------------------------   
   
--v22; colgar 24, empezar 37, forzar 42, jugar 47, regar 66, trocar 80 
-- Verbs with orthographical changes and diphthong   
mkVerbColgar :: String -> Verb   
mkVerbColgar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPColgar s s1 s2 s3 tmnp   
    where   
      s1 = dift (stem s)   
      s2 = orto4 (stem s)   
      s3 = orto4 (dift (stem s))   
   
mkVPColgar :: FourStemVP   
mkVPColgar forzar fuerz forc fuerc vp = case vp of   
    Pres Ind n p   -> mkEnd9     forz fuerz presIndEndAr n p   
    Pres Subj n p  -> mkEnd9     forc fuerc presSubjEndAr n p   
    Past Ind n p   -> mkEndings  forz pastIndEndAr n p   
    Past Subj n p  -> pastSubjAr forz n p   
    Pret n p       -> mkEnd24    forz forc pretEndAr n p   
    Fut Ind n p    -> mkEndings  forzar futIndEnd n p   
    Fut Subj n p   -> mkEndings  forz futSubjEndAr n p   
    Cond n p       -> mkEndings  forzar condEnd n p   
    Imp n p        -> mkImpEnd24 forz fuerz forc fuerc impEndAr n p
    Pass n g       -> mkPass    (forz ++ "ado") n g 
    where forz = stem forzar   
   
mkImpEnd24 :: FourStemEnd   
mkImpEnd24 forz fuerz forc fuerc ([], a, e, emos, ad, en) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> fuerz ++ a   
          P3 -> fuerc ++ e   
    Pl -> case p of   
          P1 -> forc  ++ emos   
          P2 -> forz  ++ ad   
          P3 -> fuerc ++ en   

mkEnd24 :: TwoStemEnd   
mkEnd24 colg colgu (o, as, a, amos, is, an) n p = case n of   
    Sg -> case p of   
          P1 -> colgu ++ o    
          P2 -> colg  ++ as   
          P3 -> colg  ++ a   
    Pl -> case p of   
          P1 -> colg  ++ amos   
          P2 -> colg  ++ is   
          P3 -> colg  ++ an   
   
---------------------------------------------------------------------   
   
-- conocer 25 (see group 14)   
   
---------------------------------------------------------------------   
   
-- creer 26   (see group 20)   
   
---------------------------------------------------------------------   
   
--v23; dar 27   
mkVerbDar :: String -> Verb   
mkVerbDar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPDar s tmnp   
   
mkVPDar :: String -> VPers -> String   
mkVPDar dar vp = case vp of   
    Pres Ind n p   -> mkEndings  d presIndEndDar n p   
    Pres Subj n p  -> mkNPForm presSubjDar n p   
    Past Ind n p   -> mkEndings  d pastIndEndAr n p   
    Past Subj n p  -> pastSubjEr d n p   
    Pret n p       -> mkEndings  d pretEndDar n p   
    Fut Ind n p    -> mkEndings  dar futIndEnd n p   
    Fut Subj n p   -> mkEndings  d futSubjEndEr n p   
    Cond n p       -> mkEndings  dar condEnd n p   
    Imp n p        -> mkImpEnd   d impEndDar n p
    Pass n g       -> mkPass    (d ++ "ado") n g  
    where d = stem dar   
   
presIndEndDar,presSubjDar,pretEndDar,impEndDar :: Endings   
presIndEndDar = ("oy", "as", "a", "amos", "ais", "an")   
presSubjDar   = ("d", "des", "d", "demos", "deis", "den")   
pretEndDar    = ("i", "iste", "io", "imos", "isteis", "ieron")   
impEndDar     = ([], "a", "", "emos", "ad", "en")   
   
---------------------------------------------------------------------   
   
--v24; decir 28   
-- Verb with orthographical changes and    
-- change of vowel in stem   
-- Irregular gerund and participle   
mkVerbDecir :: String -> Verb   
mkVerbDecir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregGerPart s s1 f		  
    VP tmnp -> mkVPDecir s s1 s2 s3 s4 tmnp   
    where    
      s1 = vow2 (stem s)   
      s2 = vow2 (orto3 (stem s))   
      s3 = orto s2   
      s4 = vow2 ((tk 3 s) ++ "r")   
   
mkVIIrregGerPart :: String -> String -> VImpers -> String   
mkVIIrregGerPart s s1 vim = case vim of   
    Inf    -> s   
    Ger    -> s1 ++ "iendo"   
    Part   -> if s == "contradecir" then "contradecido"    
              else (s1 ++ "ho")   
   
mkVPDecir :: String -> FourStemVP   
mkVPDecir decir dic dig dij dir vp = case vp of   
    Pres Ind n p   -> mkEnd22     dec dic dig presIndEndIr n p    
    Pres Subj n p  -> mkEndings   dig presSubjEndEr n p   
    Past Ind n p   -> mkEndings   dec pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer dij n p   
    Pret n p       -> mkEndings   dij pretEndDecir n p      
    Fut Ind n p    -> mkEndings   dir futIndEnd n p   
    Fut Subj n p   -> mkEndings   dij futSubjEndSer n p   
    Cond n p       -> mkEndings   dir condEnd n p   
    Imp n p        -> mkImpEnd28  dec dig impEndDecir n p 
    Pass n g       -> if decir == "contradecir" then mkPass "contradecido" n g    
		      else mkPass (dic ++ "ho") n g  
    where   
      dec = stem decir   
   
pretEndDecir, impEndDecir :: Endings   
pretEndDecir = ("e", "iste", "o", "imos", "isteis", "eron")  
impEndDecir  = ([], "", "a", "amos", "id", "an")   
    
mkImpEnd28 :: TwoStemEnd   
mkImpEnd28 dec dig ([], _, a, amos, id, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> tk 1 dig   
          P3 -> dig ++ a   
    Pl -> case p of   
          P1 -> dig ++ amos   
          P2 -> dec ++ id   
          P3 -> dig ++ an   
   
---------------------------------------------------------------------   
   
-- defender 29   (see group 10)   
   
---------------------------------------------------------------------   
   
-- delinquir 30  (see group 14)   
   
---------------------------------------------------------------------   
   
--v25; desosar 31, errar 41   
-- Verbs with diphthong   
mkVerbDesosar :: String -> Verb   
mkVerbDesosar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPActuar s s1 tmnp   
    where s1 = dift1 (stem s)   
   
---------------------------------------------------------------------   
   
-- dirigir 32    (see group 14)   
   
---------------------------------------------------------------------   
   
-- discernir 33  (see group 10)   
   
---------------------------------------------------------------------   
   
-- distinguir 34 (see group 14)   
   
---------------------------------------------------------------------   
   
--v26; dormir 35, sentir 76   
-- Verbs with diphthong and   
-- change of vowel in stem   
-- Irregular gerund   
mkVerbDormir :: String -> Verb   
mkVerbDormir s vf = mkStrWords $ case vf of   
    VI f    -> if suff == "morir" then mkVIIrregPartEr s (s2 ++"to") f
	       else mkVIIrregGerEr s s1 f   
    VP tmnp -> mkVPDormir s s1 s2 tmnp   
    where    
      s1 = vow2 (stem s)   
      s2 = dift (stem s)
      (_, suff) = vPrefix 5 s   
   
mkVIIrregGerEr :: String -> String -> VImpers -> String   
mkVIIrregGerEr s gerForm vim = case vim of   
    Inf    -> s   
    Ger    -> gerForm ++ "iendo"   
    Part   -> if s == "elegir" then (stem s) ++ "ido electo"
	      else (stem s) ++ "ido"
   
mkVPDormir :: ThreeStemVP   
mkVPDormir dormir durm duerm vp = case vp of   
    Pres Ind n p   -> mkEnd9 dorm duerm presIndEndIr n p   
    Pres Subj n p  -> mkEnd9 durm duerm presSubjEndEr n p   
    Past Ind n p   -> mkEndings dorm pastIndEndEr n p   
    Past Subj n p  -> if ((last durm) == '' || (last durm == 'y'))   
		      then pastSubjSer durm n p   
		      else pastSubjEr durm n p   
    Pret n p       -> if ((last durm) == '' || (last durm == 'y'))    
		      then mkEnd35 dorm durm pretEndBruir n p   
		      else mkEnd35 dorm durm pretEndEr n p   
    Fut Ind n p    -> mkEndings dormir futIndEnd n p   
    Fut Subj n p   -> if ((last durm) == '' || (last durm == 'y'))    
		      then mkEndings durm futSubjEndSer n p   
		      else mkEndings durm futSubjEndEr n p   
    Cond n p       -> mkEndings dormir condEnd n p   
    Imp n p        -> mkImpEnd24 dorm duerm durm duerm impEndIr n p 
    Pass n g       -> if morir == "morir" then mkPass (pre ++ "muerto") n g
		      else mkPass (dorm ++ "ido") n g   
    where 
      dorm = stem dormir 
      (pre, morir) = vPrefix 5 dormir   
   
mkEnd35 :: TwoStemEnd   
mkEnd35 dorm durm (o, as, a, amos, is, an) n p = case n of   
    Sg -> case p of   
          P1 -> dorm ++ o   
          P2 -> dorm ++ as   
          P3 -> durm ++ a   
    Pl -> case p of   
          P1 -> dorm ++ amos   
          P2 -> dorm ++ is   
          P3 -> durm ++ an   
   
---------------------------------------------------------------------   
   
--v27; elegir 36   
-- Verb with orthographical changes and   
-- change of vowel in the stem   
mkVerbElegir :: String -> Verb   
mkVerbElegir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregGerEr s s1 f   
    VP tmnp -> mkVPElegir s s1 s2 tmnp   
    where    
      s1 = vow2 (stem s)   
      s2 = orto s1   
   
mkVPElegir :: ThreeStemVP   
mkVPElegir elegir elig elij vp = case vp of   
    Pres Ind n p   -> mkEnd22    eleg elig elij presIndEndIr n p   
    Pres Subj n p  -> mkEndings  elij presSubjEndEr n p   
    Past Ind n p   -> mkEndings  eleg pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr elig n p   
    Pret n p       -> mkEnd35    eleg elig pretEndEr n p   
    Fut Ind n p    -> mkEndings  elegir futIndEnd n p   
    Fut Subj n p   -> mkEndings  elig futSubjEndEr n p   
    Cond n p       -> mkEndings  elegir condEnd n p   
    Imp n p        -> mkImpEnd24 eleg elig elij elij impEndIr n p
    Pass n g       -> if elegir == "elegir" 
                      then mkPass (eleg ++ "ido") n g ++ mkPass " electo" n g 
		      else mkPass (eleg ++ "ido") n g 
    where eleg = stem elegir   
   
---------------------------------------------------------------------   
   
-- empezar 37	(see group 24)   
   
---------------------------------------------------------------------   
   
--v28; encontrar 38, pensar 56   
-- Verbs with diphthong   
mkVerbEncontrar :: String -> Verb   
mkVerbEncontrar s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr s f   
    VP tmnp -> mkVPActuar s s1 tmnp   
    where s1 = dift (stem s)   
   
---------------------------------------------------------------------   
   
-- enraizar 39	(see group 11)   
   
---------------------------------------------------------------------   
   
--v29; erguir 40, seguir 75   
-- Verbs with orthographical changes and   
-- change of vowel in the stem   
mkVerbErguir :: String -> Verb   
mkVerbErguir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregGerEr s s1 f   
    VP tmnp -> mkVPErguir s s1 tmnp   
    where    
      s1 = (vow2 (orto (stem s))) ++ "u"   
   
mkVPErguir :: TwoStemVP   
mkVPErguir erguir irgu vp = case vp of   
    Pres Ind n p   -> mkEnd9 erg sig presIndEndErguir n p  
    Pres Subj n p  -> if erguir == "erguir" then mkNPForm presSubjErguir n p   
		      else mkEndings sig presSubjEndEr n p   
    Past Ind n p   -> mkEndings ergu pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr irgu n p   
    Pret n p       -> mkEnd35 ergu irgu pretEndEr n p   
    Fut Ind n p    -> mkEndings erguir futIndEnd n p   
    Fut Subj n p   -> mkEndings irgu futSubjEndEr n p   
    Cond n p       -> mkEndings erguir condEnd n p   
    Imp n p        -> if erguir == "erguir" then mkNPForm impErguir n p   
		      else if (n == Sg && p == P1) then [] 
		           else pre ++ (mkNPForm impSeguir n p)  
    Pass n g       -> mkPass (ergu ++ "ido") n g  
    where    
      ergu = stem erguir   
      erg  = tk 1 ergu
      sig  = if erguir == "erguir" then "yerg" else tk 1 irgu 
      (pre, _) = vPrefix 6 erguir
   
presIndEndErguir,presSubjErguir,impErguir,impSeguir :: Endings   
presIndEndErguir  = ("o", "ues", "ue", "uimos", "us", "uen")    
presSubjErguir    = ("yerga", "yergas", "yerga", "irgamos", "irgis", "yergan")   
impErguir         = ([], "yergue", "yerga", "yergamos", "erguid", "yergan")   
impSeguir         = ([], "sigue", "siga", "sigamos", "seguid", "sigan")   
   
---------------------------------------------------------------------   
   
-- errar  41	(see group 31)   
   
---------------------------------------------------------------------   
   
-- forzar 42	(see group 24)   
   
---------------------------------------------------------------------   
   
-- guiar  43	(see group 9)   
   
---------------------------------------------------------------------   
   
--v30; hacer 44, satisfacer 74   
-- (satisfacer extraform imp 1 pers sing)   
-- Verbs with orthographical changes and   
-- change of vowel in the stem   
mkVerbHacer :: String -> Verb   
mkVerbHacer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregPartEr s part f   
    VP tmnp -> mkVPHacer part s s1 s2 s3 tmnp   
    where   
      s1 = orto3 (stem s)   
      s2 = vow1 (stem s)   
      s3 = (tk 3 s) ++ "r"   
      part = (tk 4 s) ++ "echo"   
   
mkVPHacer :: String -> FourStemVP   
mkVPHacer partform hacer hag hic har vp = case vp of   
    Pres Ind n p   -> mkEnd1     hac sp1 presIndEndEr n p   
    Pres Subj n p  -> mkEndings  hag presSubjEndEr n p   
    Past Ind n p   -> mkEndings  hac pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr hic n p   
    Pret n p       -> mkEnd44    hic pretEndHaber n p     
    Fut Ind n p    -> mkEndings  har futIndEnd n p   
    Fut Subj n p   -> mkEndings  hic futSubjEndEr n p   
    Cond n p       -> mkEndings  har condEnd n p   
    Imp n p        -> mkImpEnd44 hac hag impEndEr n p
    Pass n g       -> mkPass     partform n g    
    where   
      hac = stem hacer   
      sp1 = hag ++ "o"   
   
mkEnd44 :: OneStemEnd   
mkEnd44 hic (o, as, a, amos, is, an) n p = case n of   
    Sg -> case p of   
          P1 -> hic ++ o   
          P2 -> hic ++ as   
          P3 -> orto1 hic ++ a   
    Pl -> case p of   
          P1 -> hic ++ amos   
          P2 -> hic ++ is   
          P3 -> hic ++ an   
   
mkImpEnd44 :: TwoStemEnd   
mkImpEnd44 hac hag ([], e, a, amos, ed, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> if hac == "satisfac" then orto1 hac ++ " satisface"
		else orto1 hac
          P3 -> hag ++ a   
    Pl -> case p of   
          P1 -> hag ++ amos   
          P2 -> hac ++ ed   
          P3 -> hag ++ an   
   
---------------------------------------------------------------------   
   
--v31; influir 45   
-- Verb with diphthong   
mkVerbInfluir :: String -> Verb   
mkVerbInfluir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIInfluir s f   
    VP tmnp -> mkVPDormir s s1 s1 tmnp   
    where s1 = dift1 (stem s)    
   
mkVIInfluir :: String -> VImpers -> String   
mkVIInfluir s vim = case vim of   
    Inf    -> s   
    Ger    -> stem s  ++ "yendo"   
    Part   -> stem s  ++ "ido"   
   
---------------------------------------------------------------------   
   
-- ir    46	(see group 1)   
   
---------------------------------------------------------------------   
   
-- jugar 47	(see group 24)   
   
---------------------------------------------------------------------   
   
-- lucir 48	(see group 14)   
   
---------------------------------------------------------------------   
   
--v32; mecer 49, yacer 85, zurcir 86   
-- Verbs with orthographical changes   
mkVerbMecer :: String -> Verb   
mkVerbMecer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPAsir s s1 tmnp   
    where   
      (_, yacer) = vPrefix 5 s   
      s1 = if yacer == "yacer" then orto2 (stem s)   
           else orto1 (stem s)   
   
---------------------------------------------------------------------   
   
-- mover 50	(see group 10)   
   
---------------------------------------------------------------------   
   
--v33; or 51                 
mkVerbOr :: String -> Verb   
mkVerbOr s vf = mkStrWords $ case vf of   
    VI f    -> mkVICaer s f   
    VP tmnp -> mkVPOr s s1 s2 s3 tmnp   
    where    
      (pre, oir) = vPrefix 3 s   
      s1 = (stem s) ++ "ig"   
      s2 = (stem s) ++ "y"   
      s3 = (stem s) ++ "ir"   
    
mkVPOr :: FourStemVP   
mkVPOr or oig oy oir vp = case vp of   
    Pres Ind n p   -> mkEnd22     o oy oig presIndEndOr n p    
    Pres Subj n p  -> mkEndings   oig presSubjEndEr n p   
    Past Ind n p   -> mkEndings   o pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer oy n p   
    Pret n p       -> mkEnd35     o oy pretEndOr n p      
    Fut Ind n p    -> mkEndings   oir futIndEnd n p   
    Fut Subj n p   -> mkEndings   oy futSubjEndSer n p   
    Cond n p       -> mkEndings   oir condEnd n p   
    Imp n p        -> mkImpEnd61  o oig oy impEndOr n p   
    Pass n g       -> mkPass     (o ++ "do") n g     
    where   
      o = stem or   
   
presIndEndOr,pretEndOr,impEndOr :: Endings   
presIndEndOr = ("o", "es", "e", "mos", "s", "en")   
pretEndOr    = ("", "ste", "", "mos", "steis", "eron")   
impEndOr     = ([], "e", "a", "amos", "d", "an")   
   
---------------------------------------------------------------------   
   
--v34; oler 52   
-- Verb with diphthong   
mkVerbOler :: String -> Verb   
mkVerbOler s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPProhibir s s1 tmnp   
    where s1 = dift1 (stem s)   
   
---------------------------------------------------------------------   
   
-- pagar   53	(see group 17)   
   
---------------------------------------------------------------------   
   
-- parecer 54	(see group 14)   

---------------------------------------------------------------------   
   
--v35; acaecer 54, acontecer 54, aplacer 57   
-- Defect verbs!!!    
mkVerbAcaecer :: String -> Verb   
mkVerbAcaecer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> if s == "aplacer" then mkVPAplacer s tmnp    
	       else mkVPAcaecer s s1 tmnp   
    where s1 = orto (stem s)   
   
mkVPAcaecer :: TwoStemVP   
mkVPAcaecer acaecer acaezc vp = case vp of   
    Pres Ind n p   -> mkEnd54 acaec "e" "en" n p                      
    Pres Subj n p  -> mkEnd54 acaezc "a" "an" n p     
    Past Ind n p   -> mkEnd54 acaec "a" "an" n p   
    Past Subj n p  -> pastSubjAcaecer acaec n p   
    Pret n p       -> mkEnd54 acaec "i" "ieron" n p     
    Fut Ind n p    -> mkEnd54 acaecer "" "n" n p    
    Fut Subj n p   -> mkEnd54 acaec "iere" "ieren" n p      
    Cond n p       -> mkEnd54 acaecer "a" "an" n p    
    Imp n p        -> mkEnd54 acaezc "a" "an" n p    
    Pass n g       -> mkPass (acaec ++ "ido") n g     
    where acaec = stem acaecer   
   
pastSubjAcaecer :: String -> Number -> Person -> String   
pastSubjAcaecer s n p = acaeciera ++ " " ++ acaeciese where   
  acaeciera = mkEnd54 s "iera" "ieran" n p   
  acaeciese = mkEnd54 s "iese" "iesen" n p   
   
mkEnd54 :: String -> String -> String -> Number -> Person -> String   
mkEnd54 acaec sg3 pl3 n p = case p of   
				 P1 -> []   
				 P2 -> []   
			         P3 -> if n == Sg then acaec ++ sg3    
                                       else acaec ++ pl3   
   
mkVPAplacer :: String -> VPers -> String   
mkVPAplacer s vp = case vp of   
    Pres Ind n p   -> mkEnd54 (stem s)  "e" "en" n p    
    Pres Subj n p  -> []   
    Past Ind n p   -> mkEnd54 (stem s) "a" "an" n p   
    Past Subj n p  -> []   
    Pret n p       -> []   
    Fut Ind n p    -> []   
    Fut Subj n p   -> []   
    Cond n p       -> []   
    Imp n p        -> []
    Pass n g       -> []      
   
---------------------------------------------------------------------   
   
--v36; pedir 55, reir 68   
-- Verb with change of vowel in the stem   
mkVerbPedir :: String -> Verb   
mkVerbPedir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregGerEr s s1 f   
    VP tmnp -> mkVPDormir s s1 s1 tmnp   
    where    
      s1 = vow2 (stem s)   
   
---------------------------------------------------------------------   
   
-- pensar  56	(see group 38)   
   
---------------------------------------------------------------------   
   
-- placer 57 (see group 14)

---------------------------------------------------------------------   
   
--v37; abarse  57 - Defect verbs   
mkVerbAbarse :: String -> Verb   
mkVerbAbarse s = mkVerbDefectAbarse s `except` [(VI Inf, s)]    
   
mkVerbDefectAbarse :: String -> Verb   
mkVerbDefectAbarse s vf = mkStrWords $ case vf of   
    VI f    -> mkVIAr (stem s) f               
    VP tmnp -> mkVPAbarse s tmnp   
    
mkVPAbarse :: String -> VPers -> String   
mkVPAbarse abarse vp = case vp of   
    Pres Ind n p   -> []   
    Pres Subj n p  -> []   
    Past Ind n p   -> []   
    Past Subj n p  -> []   
    Pret n p       -> []   
    Fut Ind n p    -> []   
    Fut Subj n p   -> []   
    Cond n p       -> []   
    Imp n p        -> case p of    
                        P1 -> []   
                        P2 -> if n == Sg then "aba" else "abad"   
                        P3 -> []
    Pass n g       -> []       
   
---------------------------------------------------------------------   
   
--v38; poder 58, querer 64   
-- Verbs with diphthong   
mkVerbPoder :: String -> Verb   
mkVerbPoder s vf = mkStrWords $ case vf of   
    VI f    -> if s == "poder" then mkVIIrregGerEr s s2 f   
	    else mkVIEr s f   
    VP tmnp -> mkVPPoder s s1 s2 s3 tmnp   
    where    
      s1 = dift (stem s)   
      s2 = if s == "poder" then vow2 (stem s)   
	   else (tk 2 s1) ++ "s"   
      s3 = dropSecondLast s   
    
mkVPPoder :: FourStemVP   
mkVPPoder poder pued pud podr vp = case vp of   
    Pres Ind n p   -> mkEnd9     pod pued presIndEndEr n p    
    Pres Subj n p  -> mkEnd9     pod pued presSubjEndEr n p   
    Past Ind n p   -> mkEndings  pod pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr pud n p   
    Pret n p       -> mkEndings  pud pretEndHaber n p    
    Fut Ind n p    -> mkEndings  podr futIndEnd n p   
    Fut Subj n p   -> mkEndings  pud futSubjEndEr n p   
    Cond n p       -> mkEndings  podr condEnd n p   
    Imp n p        -> mkImpEnd9  pod pued impEndEr n p
    Pass n g       -> mkPass    (pod ++ "ido") n g       
    where   
      pod = stem poder   
   
---------------------------------------------------------------------   
   
-- poner    60	(see group 4)   
   
---------------------------------------------------------------------   
   
--v40; predecir 61   
-- Verb with orthographical changes and   
-- change of vowel in the stem   
mkVerbPredecir :: String -> Verb   
mkVerbPredecir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregGerPart s s2 f   
    VP tmnp -> mkVPPredecir s s1 s2 s3 tmnp   
    where    
      s1 = vow2 (orto3 (stem s))   
      s2 = vow2 (stem s)   
      s3 = orto s1   
   
mkVPPredecir :: FourStemVP   
mkVPPredecir predecir predig predic predij vp = case vp of   
    Pres Ind n p   -> mkEnd22     predec predic predig presIndEndIr n p    
    Pres Subj n p  -> mkEndings   predig presSubjEndEr n p   
    Past Ind n p   -> mkEndings   predec pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer predij n p   
    Pret n p       -> mkEndings   predij pretEndDecir n p      
    Fut Ind n p    -> mkEndings   predecir futIndEnd n p   
    Fut Subj n p   -> mkEndings   predij futSubjEndSer n p   
    Cond n p       -> mkEndings   predecir condEnd n p   
    Imp n p        -> mkImpEnd61  predec predig predic impEndIr n p  
    Pass n g       -> if predecir == "contradecir" then mkPass "contradecido" n g
		      else mkPass (predic ++ "ho") n g     
    where   
      predec = stem predecir   
   
mkImpEnd61 :: ThreeStemEnd   
mkImpEnd61 predec predig predic ([], e, a, amos, id, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> predic ++ e   
          P3 -> predig ++ a   
    Pl -> case p of   
          P1 -> predig ++ amos   
          P2 -> predec ++ id   
          P3 -> predig ++ an   
   
---------------------------------------------------------------------   
   
--v41; producir 62   
-- Verb with orthographical changes   
mkVerbProducir :: String -> Verb   
mkVerbProducir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPProducir s s1 s2 tmnp   
    where    
      s1 = orto (stem s)   
      s2 = orto5 (stem s)   
   
mkVPProducir :: ThreeStemVP   
mkVPProducir producir produzc produj vp = case vp of   
    Pres Ind n p   -> mkEnd24     produc produzc presIndEndIr n p   
    Pres Subj n p  -> mkEndings   produzc presSubjEndEr n p   
    Past Ind n p   -> mkEndings   produc pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer produj n p   
    Pret n p       -> mkEndings   produj pretEndDecir n p    
    Fut Ind n p    -> mkEndings   producir futIndEnd n p   
    Fut Subj n p   -> mkEndings   produj futSubjEndSer n p   
    Cond n p       -> mkEndings   producir condEnd n p   
    Imp n p        -> mkImpEnd3   produc produzc impEndIr n p
    Pass n g       -> mkPass     (produc ++ "ido") n g       
    where    
      produc = stem producir   
   
---------------------------------------------------------------------   
   
--v42; prohibir 63, reunir 69   
-- Verbs with an accent   
mkVerbProhibir :: String -> Verb   
mkVerbProhibir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPProhibir s s1 tmnp   
    where s1 = accent (stem s)   
   
mkVPProhibir :: TwoStemVP   
mkVPProhibir prohibir prohb vp = case vp of   
    Pres Ind n p   -> if erVerb prohibir   
                      then mkEnd9 prohib prohb presIndEndEr n p   
                      else mkEnd9 prohib prohb presIndEndIr n p   
    Pres Subj n p  -> mkEnd9      prohib prohb presSubjEndEr n p   
    Past Ind n p   -> mkEndings   prohib pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr  prohib n p   
    Pret n p       -> mkEndings   prohib pretEndEr n p   
    Fut Ind n p    -> mkEndings   prohibir futIndEnd n p   
    Fut Subj n p   -> mkEndings   prohib futSubjEndEr n p   
    Cond n p       -> mkEndings   prohibir condEnd n p   
    Imp n p        -> if erVerb   prohibir   
                      then mkImpEnd9 prohib prohb impEndEr n p   
                      else mkImpEnd9 prohib prohb impEndIr n p   
    Pass n g       -> mkPass     (prohib ++ "ido") n g    
    where prohib = stem prohibir   
   
---------------------------------------------------------------------   
   
-- querer 64	(see group 58)   
   
---------------------------------------------------------------------   
   
--v43; raer 65   
mkVerbRaer :: String -> Verb   
mkVerbRaer s vf = mkStrWords $ case vf of   
    VI f    -> mkVICaer s f   
    VP tmnp -> mkVPRaer s s1 s2 tmnp    
    where   
      s1 = (stem s) ++ "ig"   
      s2 = stem s ++ "y"   
   
mkVPRaer :: ThreeStemVP    
mkVPRaer raer raig ray vp = case vp of   
    Pres Ind n p   -> mkEndings   ra presIndEndRaer n p    
    Pres Subj n p  -> mkEndings   raig presSubjEndEr n p   
    Past Ind n p   -> mkEndings   ra pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer ray n p   
    Pret n p       -> mkEnd35     ra ray pretEndOr n p    
    Fut Ind n p    -> mkEndings   raer futIndEnd n p   
    Fut Subj n p   -> mkEndings   ray futSubjEndSer n p   
    Cond n p       -> mkEndings   raer condEnd n p   
    Imp n p        -> mkImpEnd3   ra raig impEndEr n p  
    Pass n g       -> mkPass     (ra ++ "do") n g     
    where ra = stem raer   
   
presIndEndRaer :: Endings   
presIndEndRaer = ("o raigo", "es", "e", "emos", "is", "en")   
   
---------------------------------------------------------------------   
   
-- regar 66	(see group 24)   
   
---------------------------------------------------------------------   
   
--v44; rer  67   
-- Verb with a change of vowel in the stem   
mkVerbRer :: String -> Verb   
mkVerbRer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIRer s s2 f   
    VP tmnp -> mkVPRer s s1 s2 s3 tmnp   
    where    
      s1 = vow3 (stem s)   
      s2 = vow2 (stem s)   
      s3 = (stem s) ++ "ir"   
    
mkVIRer :: String -> String -> VImpers -> String   
mkVIRer s s1 vim = case vim of   
    Inf    -> s   
    Ger    -> s1 ++ "endo"   
    Part   -> if s == "frer" then (stem s ++ "do") ++ " frito"
	      else (stem s ++ "do")   
   
mkVPRer :: FourStemVP   
mkVPRer rer r ri reir vp = case vp of   
    Pres Ind n p   -> mkEnd9      re r presIndEndOr n p    
    Pres Subj n p  -> mkEnd9      ri r presSubjEndEr n p   
    Past Ind n p   -> mkEndings   re pastIndEndEr n p   
    Past Subj n p  -> pastSubjSer ri n p   
    Pret n p       -> mkEnd35     re ri pretEndOr n p      
    Fut Ind n p    -> mkEndings   reir futIndEnd n p   
    Fut Subj n p   -> mkEndings   ri futSubjEndSer n p   
    Cond n p       -> mkEndings   reir condEnd n p   
    Imp n p        -> mkImpEnd67  re r ri impEndOr n p
    Pass n g       -> if rer == "frer" then (mkPass (re ++ "do") n g) ++ (mkPass " frito" n g)
		      else mkPass (re ++ "do") n g      
    where   
      re = stem rer   
   
mkImpEnd67 :: ThreeStemEnd   
mkImpEnd67 re r ri ([], e, a, amos, id, an) n p = case n of   
    Sg -> case p of   
          P1 -> []   
          P2 -> r ++ e   
          P3 -> r ++ a   
    Pl -> case p of   
          P1 -> ri ++ amos   
          P2 -> re ++ id   
          P3 -> r ++ an   
   
---------------------------------------------------------------------   
   
-- reir  68	(see group 55)   
   
---------------------------------------------------------------------   
   
-- reunir 69	(see group 9)   
   
---------------------------------------------------------------------   
   
-- roer   70 	(see group 20)   
   
---------------------------------------------------------------------   
   
-- saber  71	(see group 3)   
   
---------------------------------------------------------------------   
   
-- sacar  72	(see group 17)   
   
---------------------------------------------------------------------   
   
--v45; salir  73, valer 81   
mkVerbSalir :: String -> Verb   
mkVerbSalir s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPSalir s s1 s2 tmnp   
    where    
      s1 = (stem s) ++ "g"         
      s2 = (stem s) ++ "dr"   
   
mkVPSalir :: ThreeStemVP   
mkVPSalir salir salg saldr vp = case vp of   
    Pres Ind n p   -> if irVerb    salir   
		      then mkEnd24 sal salg presIndEndIr n p   
		      else mkEnd24 sal salg presIndEndEr n p   
    Pres Subj n p  -> mkEndings    salg presSubjEndEr n p   
    Past Ind n p   -> mkEndings    sal pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr   sal n p   
    Pret n p       -> mkEndings    sal pretEndEr n p   
    Fut Ind n p    -> mkEndings    saldr futIndEnd n p   
    Fut Subj n p   -> mkEndings    sal futSubjEndEr n p   
    Cond n p       -> mkEndings    saldr condEnd n p   
    Imp n p        -> if irVerb    salir    
		      then mkImpEnd3 sal salg impEndDecir n p   
		      else mkImpEnd3 sal salg impEndEr n p  
    Pass n g       -> mkPass      (sal ++ "ido") n g  
    where sal = stem salir   
   
---------------------------------------------------------------------   
   
-- satisfacer 74	(see group 44)   
   
---------------------------------------------------------------------   
   
-- seguir     75	(see group 40)   
   
---------------------------------------------------------------------   
   
-- sentir     76	(see group 35)   
   
---------------------------------------------------------------------   
   
--v46; soler  77   
-- Defect verb   
mkVerbSoler :: String -> Verb   
mkVerbSoler s vf = mkStrWords $ case vf of   
    VI f    -> mkVIDefect s f   
    VP tmnp -> mkVPSoler s s1 tmnp   
    where s1 = dift (stem s)   
      
mkVPSoler :: TwoStemVP   
mkVPSoler soler suel vp = case vp of   
    Pres Ind n p   -> mkEnd9     sol suel presIndEndEr n p   
    Pres Subj n p  -> mkEnd9     sol suel presSubjEndEr n p   
    Past Ind n p   -> mkEndings  sol pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr sol n p   
    Pret n p       -> mkEndings  sol pretEndEr n p   
    Fut Ind n p    -> []   
    Fut Subj n p   -> []   
    Cond n p       -> []   
    Imp n p        -> []   
    Pass n g       -> []
    where sol = stem soler   
   
---------------------------------------------------------------------   
   
-- taer  78	(see group 18)   
   
---------------------------------------------------------------------   
   
-- traer  79	(see group 20)   
   
---------------------------------------------------------------------   
   
-- trocar 80	(see group 24)   
   
---------------------------------------------------------------------   
   
-- valer  81	(see group 73)   
   
---------------------------------------------------------------------   
   
-- venir  82	(see group 4)   
   
---------------------------------------------------------------------   
   
--v47; ver 83   
mkVerbVer :: String -> Verb   
mkVerbVer s vf = mkStrWords $ case vf of   
    VI f    -> mkVIIrregPartEr s (pre ++ "visto") f   
    VP tmnp -> mkVPVer s s1 tmnp   
    where
    (pre, ver) = vPrefix 3 s 
    s1 = tk 1 s   
   
mkVPVer :: TwoStemVP   
mkVPVer ver ve vp = case vp of   
    Pres Ind n p   -> mkEnd24    v ve presIndEndVer n p   
    Pres Subj n p  -> mkEndings  ve presSubjEndEr n p   
    Past Ind n p   -> mkEndings  ve pastIndEndEr n p   
    Past Subj n p  -> pastSubjEr v n p   
    Pret n p       -> mkEndings  v pretEndDar n p             
    Fut Ind n p    -> mkEndings  ver futIndEnd n p   
    Fut Subj n p   -> mkEndings  v futSubjEndEr n p   
    Cond n p       -> mkEndings  ver condEnd n p   
    Imp n p        -> mkImpEnd3  v ve impEndEr n p  
    Pass n g       -> mkPass    (pre ++ "visto") n g  
    where 
    v = stem ver  
    (pre, s) = vPrefix 3 ver 
   
presIndEndVer :: Endings   
presIndEndVer = ("o", "es", "e", "emos", "eis", "en")   
   
---------------------------------------------------------------------   
   
-- volver 84	(see group 10)   
   
---------------------------------------------------------------------   
   
-- yacer  85	(see group 49)   
   
---------------------------------------------------------------------   
   
-- zurcir 86	(see group 49)   
   
---------------------------------------------------------------------   
   
-- amar   87	(passive forms)   
   
---------------------------------------------------------------------   
   
-- levantarse 88(reflexive verbs) 	   
   
---------------------------------------------------------------------   
   
-- llover 89	(impersonal verbs)   
   
----------------------------------------------------------------------   

--v48; prender 6, imprimir 7 
-- double participle
mkVerbPrender :: String -> Verb   
mkVerbPrender s vf = mkStrWords $ case vf of   
    VI f    -> mkVIPrender s f   
    VP tmnp -> if erVerb s then mkVPEr s tmnp
	       else mkVPIr s tmnp
   
mkVIPrender :: String -> VImpers -> String   
mkVIPrender s vim = case vim of   
    Inf    -> s   
    Ger    -> (stem s) ++ "iendo"   
    Part   -> if s == "prender" then (stem s) ++ "ido" ++ " presto"  
	      else (stem s) ++ "ido" ++ " impreso"

----------------------------------------------------------------------

--v49; llover 89
--Impersonal verbs

mkVerbLlover :: String -> Verb   
mkVerbLlover s vf = mkStrWords $ case vf of   
    VI f    -> mkVIEr s f   
    VP tmnp -> mkVPLlover s s1 tmnp
	where s1 = dift (stem s)         
	   
mkVPLlover :: TwoStemVP   
mkVPLlover llover lluev vp = case vp of   
    Pres Ind n p   -> mkEnd89 lluev   "e"    n p                      
    Pres Subj n p  -> mkEnd89 lluev   "a"    n p     
    Past Ind n p   -> mkEnd89 llov    "a"   n p   
    Past Subj n p  -> pastSubjLlover llov    n p   
    Pret n p       -> mkEnd89 llov    "i"   n p     
    Fut Ind n p    -> mkEnd89 llover  ""    n p    
    Fut Subj n p   -> mkEnd89 llov    "iere" n p      
    Cond n p       -> mkEnd89 llover  "a"   n p    
    Imp n p        -> []
    Pass n g       -> []
    where llov  = stem llover
	     
pastSubjLlover :: String -> Number -> Person -> String   
pastSubjLlover s n p = lloviera ++ " " ++ lloviese where   
  lloviera = mkEnd89 s "iera" n p   
  lloviese = mkEnd89 s "iese" n p 

mkEnd89 :: String -> String -> Number -> Person -> String   
mkEnd89 llov sg3 n p = case p of   
				 P1 -> []   
				 P2 -> []   
			         P3 -> if n == Sg then llov ++ sg3 else []


----------------------------------------------------------------------

-- closed classes 
 
---------------------------------------------------------------------- 
   
-- Articles   
   
artIndef :: String -> Article
artIndef s (ArtF n ) = mkStrWords $ case n of     
	Sg	-> s
	Pl 	-> if s == "un" then "unos" else "unas" 

artDef :: String -> Article
artDef s (ArtF n ) = mkStrWords $ case n of   
	Sg	-> s		   
	Pl 	-> if s == "la" then "las" else "los"

----------------------------------------------------------------------   
 
-- Pronouns    
 
-- Personal pronouns   
mkPersPron :: PersPron   
mkPersPron (PPF c n p g) = mkStrWords $ case c of   
    Sub      ->  case n of      
		 Sg -> case p of   
		       P1 -> "yo"   
		       P2 -> "t"   
		       P3 -> case g of   
			     Masc1 -> "l"   
			     Fem1   -> "ella"   
			     Neutr -> "ello"   
		 Pl -> case p of   
		       P1 -> if g == Fem1 then "nosotras" else "nosotros"   
		       P2 -> if g == Fem1 then "vosotras" else "vosotros"   
		       P3 -> if g == Fem1 then "ellas ustedes vds. uds."   
			     else "ellos ustedes vds. uds."  
    PrepExpr  ->  case n of      
		 Sg -> case p of   
		       P1 -> "m"   
		       P2 -> "t"   
		       P3 -> case g of   
			     Masc1 -> "l"   
			     Fem1   -> "ella"   
			     Neutr -> "ello"   
		 Pl -> case p of   
		       P1 -> if g == Fem1 then "nosotras" else "nosotros"   
		       P2 -> if g == Fem1 then "vosotras" else "vosotros"   
		       P3 -> if g == Fem1 then "ellas ustedes vds. uds."   
			     else "ellos ustedes vds. uds."   
    DirObj   -> case n of      
		 Sg -> case p of   
		       P1 -> "me"   
		       P2 -> "te"   
		       P3 -> case g of   
			     Masc1 -> "le"   
			     Fem1   -> "la"   
			     Neutr -> "lo"   
		 Pl -> case p of   
		       P1 -> "nos"   
		       P2 -> "vos"   
		       P3 -> case g of   
			     Masc1 -> "les"   
			     Fem1   -> "las"   
			     Neutr -> "los"   
    IndirObj -> case n of      
		 Sg -> case p of   
		       P1 -> "me"   
		       P2 -> "te"   
		       P3 -> "le se"   
		 Pl -> case p of   
		       P1 -> "nos"   
		       P2 -> "vos"   
		       P3 -> "les se"   
 
-- Reflexive pronouns 
mkReflPron :: ReflPron  
mkReflPron (RPF n p) = mkStrWords $ case n of  
    Sg -> case p of  
	  P1 -> "me"   
	  P2 -> "te"   
	  P3 -> "se"    
    Pl -> case p of	    
	  P1 -> "nos"   
	  P2 -> "os"   
	  P3 -> "se"  
 
-- Possessive pronouns  
mkPossPron :: PossPron  
mkPossPron (PossPF m n p g n1) = mkStrWords $ case m of          
    Unstressed -> case n of  
		  Sg -> case p of  
			P1 -> mkNumber n1 "mi"  
			P2 -> mkNumber n1 "tu"  
			P3 -> mkNumber n1 "su"  
		  Pl -> case p of  
			P1 -> case g of   
			      Masc -> mkNumber n1 "nuestro"  
			      Fem  -> mkNumber n1 "nuestra"  
			P2 -> case g of   
			      Masc -> mkNumber n1 "vuestro"  
			      Fem  -> mkNumber n1 "vuestra"  
			P3 -> mkNumber n1 "su"  
    Stressed   -> case n of  
		  Sg -> case p of  
			P1 -> case g of   
			      Masc -> mkNumber n1 "mo"  
			      Fem  -> mkNumber n1 "ma"  
			P2 -> case g of   
			      Masc -> mkNumber n1 "tuyo"  
			      Fem  -> mkNumber n1 "tuya"  
			P3 -> case g of   
			      Masc -> mkNumber n1 "suyo"  
			      Fem  -> mkNumber n1 "suya"  
		  Pl -> case p of  
			P1 -> case g of   
			      Masc -> mkNumber n1 "nuestro"  
			      Fem  -> mkNumber n1 "nuestra"  
			P2 -> case g of   
			      Masc -> mkNumber n1 "vuestro"  
			      Fem  -> mkNumber n1 "vuestra"  
			P3 -> case g of   
			      Masc -> mkNumber n1 "suyo"  
			      Fem  -> mkNumber n1 "suya" 
 
-- Demonstrative pronouns  
mkDemoPron :: String -> DemoPron  
mkDemoPron s (DPF n g) = mkStrWords $ case n of  
    Sg -> case g of  
	  Masc1 -> if s == "aquel" then s ++ " " ++ (accent s)  
		    else s ++ " " ++ as ++ "e"  
	  Fem1   -> sc ++ "a" ++ " " ++ as ++ "a"  
	  Neutr -> sc ++ "o"   
    Pl -> case g of	    
	  Masc1 -> sc ++ "os" ++ " " ++ as ++ "os"  
	  Fem1   -> sc ++ "as" ++ " " ++ as ++ "as"  
	  Neutr -> sc ++ "os"  
    where   
      sc = if s == "aquel" then s ++ "l"   
           else tk 1 s  
      as = accent sc	 
 
-- Relative pronouns    
mkRelPron :: String -> RelPron  
mkRelPron s (RelPF n g) = mkStrWords $ case s of   
    "que"    -> s  
    "quien"  -> mkNumber n s  
    "cual"   -> mkNumber n s  
    "cuyo"   -> mkNumber n $ case g of   
		Masc-> s  
		Fem  -> (tk 1 s) ++ "a"  
    "donde"  -> s  
    "cuanto" -> mkNumber n s  
 
-- Interrogative pronouns   
mkIntPron :: String -> IntPron  
mkIntPron s (IPF n g) = mkStrWords $ case s of   
    "qu"    -> s  
    "quin"  -> mkPronNumber n s  
    "cal"   -> mkPronNumber n s  
    "cunto" -> mkPronNumber n $ case g of   
		Masc -> s  
		Fem  -> (tk 1 s) ++ "a"  
    "cmo"   -> s  
    _        -> s

mkPronNumber :: Number -> String -> String   
mkPronNumber n s = case n of   
  Sg -> s   
  Pl -> if isVowel (last s) then s ++ "s" else s ++ "es"  
 
-- Indefinite pronouns   
mkIndefPron1 :: String -> IndefPron1  
mkIndefPron1 s (UniPF n g) = mkStrWords $ mkPronNumber n $ case g of   
		 Masc -> if s == "alguno" then s ++ " algn" else s 
		 Fem  -> (tk 1 s) ++ "a" 
 
---------------------------------------------------------------------
   
mkNumbers :: String -> Numb
mkNumbers s (Cardinal g) = if (suffix == "ientos") then mkStrWords $ 
			  case g of 
			  Masc -> s
			  Fem  -> (tk 2 s) ++ "as"
			  else mkStrWords $ case s of 
			 "uno"       -> case g of 
				        Masc -> s ++ " un"
			       	        Fem  -> (tk 1 s) ++ "a"
		      	 "veintiuno" -> case g of 
				        Masc -> s ++ " veinten"
			       	        Fem  -> (tk 1 s) ++ "a"
			 "ciento"    -> case g of 
				        Masc -> s ++ " cien" 
				        Fem  -> (tk 1 s) ++ "a"
			 _           -> s
			 where (_, suffix) = vPrefix 6 s 

mkOrder :: String -> Order
mkOrder s (Ordinal n g) = mkStrWords $ mkNumber n $ case s of
			"primero"    -> case g of 
				        Masc -> case n of 
						Sg -> s ++ " primer" 
						Pl -> s
				        Fem  -> (tk 1 s) ++ "a"
			"tercero"    -> case g of 
				        Masc -> case n of 
						Sg -> s ++ " tercer" 
						Pl -> s
				        Fem  -> (tk 1 s) ++ "a"
			_            -> case g of 
				        Masc -> s  
				        Fem  -> (tk 1 s) ++ "a"

----------------------------------------------------------------------    
  
-- generics for Spanish   
   
isVowel  c = elem c "aAeEiIoOuUyY"   
accVowel c = elem c ""		-- does  even exist?   
   
ifVowel c d e = if isVowel c then d else e   
   
-- The stem vowel is the last vowel (or diphtong) in the stem   
findStemVowel :: String -> (String, String, String)   
findStemVowel sprick = (reverse rps, reverse i, reverse kc) where   
  (kc, irps) = break isVowel $ reverse sprick   
  (i,   rps) = span  isVowel $ irps   
   
-- The stemVowel is the last vowel in the stem   
-- (or the last vowel in the diphtong)   
stemVowel :: String -> (String, String, String)   
stemVowel sprick = (reverse rps, reverse i, reverse kc) where   
  (kc, irps) = break isVowel $ reverse sprick   
  i = take 1 irps   
  rps = drop 1 irps   

-- The stemVowel is the last vowel in the stem   
-- (or the last vowel in the diphtong)   
firstVowel :: String -> (String, String, String)   
firstVowel joven = (j, o, ven) where 
  nevoj     = reverse joven  
  (j, oven) = break isVowel $ reverse nevoj
  o         = take 1 oven   
  ven       = drop 1 oven   
   
dropSecondLast :: String -> String   
dropSecondLast s = tk 2 s ++ [last s]   
   
arVerb,erVerb,irVerb :: String -> Bool   
arVerb s = (drop (length s - 2) s) == "ar"   
erVerb s = (drop (length s - 2) s) == "er"   
irVerb s = (drop (length s - 2) s) == "ir"   
   
sing3,plur3 :: Number -> Person -> Bool   
sing3 n p = n == Sg && p == P3   
plur3 n p = n == Pl && p == P3   
   
stem :: String -> String   
stem s = tk 2 s   
   
vPrefix :: Int -> String -> (String, String)   
vPrefix i s = (pre,word) where   
    pre  = if i <= (length s) then tk i s
	   else "" 
    word = if i <= (length s) then drop(length s - i) s 
	   else ""
   
-- Accent in the stem   
accent :: String -> String   
accent actuar = act ++ mkAcc u ++ ar where   
    (act,u,ar) = stemVowel actuar   
    mkAcc v = case v of   
	      "i" -> ""   
	      "u" -> ""   
	      "e" -> ""   
   
-- Diphthong in the stem   
dift,dift1,dift2 :: String -> String   
dift adquirir    = adqu ++ mkDift i ++ rir where   
    (adqu,i,rir) = stemVowel adquirir   
    mkDift v = case v of   
	       "e" -> "ie"   
	       "i" -> "ie"   
	       "o" -> "ue"   
	       "u" -> "ue"  
dift1 adquirir   = adqu ++ mkDift i ++ rir where   
    (adqu,i,rir) = stemVowel adquirir   
    mkDift v = case v of   
               "e" -> "ye"   
	       "o" -> "hue" 
  	       "u" -> "uy"   
	       "" -> "y" 
dift2 adquirir   = adqu ++ mkDift i ++ rir where   
    (adqu,i,rir) = stemVowel adquirir   
    mkDift v = "e"   
   
-- Orthographical changes of the stem   
orto,orto1,orto2,orto3,orto4,orto5 :: String -> String   
orto as = a ++ s  where   
     a = if last as == 'u' then tk 2 as   
         else tk 1 as   
     s = case (last as) of   
        's' -> "sg"   
        'g' -> "j"   
        'c' -> "zc"   
        'u' -> case (last (tk 1 as)) of   
                  'q' -> "c"   
                  'g' -> "g"   
orto1 mec = me ++ c where   
    me = tk 1 mec   
    c = "z"   
orto2 yac = ya ++ c where   
    ya = tk 1 yac   
    c = "zg"   
orto3 hac = ha ++ c where   
    ha = tk 1 hac   
    c = "g"   
orto4 forz = for ++ z  where   
     for = tk 1 forz   
     z = case (last forz) of   
        'z' -> "c"   
        'g' -> "gu"   
        'c' -> "qu"   
	'u' -> ""   
orto5 produc = produ ++ c where   
    produ = tk 1 produc   
    c = "j"   
   
-- Change of vowel in the stem   
vow,vow1,vow2,vow3,vow4 :: String -> String   
vow hab = h ++ chVow a ++ b where   
    (h,a,b) = stemVowel hab   
    chVow v = "u"   
vow1 hac = h ++ chVow a ++ c where   
    (h,a,c) = stemVowel hac   
    chVow v = "i"   
vow2 dorm = d ++ chVow o ++ rm where   
    (d,o,rm) = stemVowel dorm   
    chVow v = case v of   
	      "o" -> "u"   
	      "e" -> "i"   
	      "i" -> "y"   
vow3 reir = r ++ chVow e ++ ir where   
    (r,e,ir) = stemVowel reir   
    chVow v = ""   
vow4 pudr = p ++ chVow u ++ dr where   
    (p,u,dr) = stemVowel pudr   
    chVow v = "o" 
 
 

